@@ -3,7 +3,7 @@ extern crate rl_ball_sym;
 
 mod utils;
 
-use cpython::{exc, py_fn, py_module_initializer, PyBool, PyDict, PyErr, PyFloat, PyInt, PyObject, PyResult, PyTuple, Python, PythonObject};
+use cpython::{exc, py_fn, py_module_initializer, ObjectProtocol, PyBool, PyDict, PyErr, PyFloat, PyInt, PyObject, PyResult, PyTuple, Python, PythonObject};
 use dubins_paths::path_sample_many;
 use rl_ball_sym::simulation::{
     ball::{Ball, BallPrediction},
@@ -29,7 +29,8 @@ py_module_initializer!(virxrlru, |py, m| {
     m.add(py, "load_dropshot", py_fn!(py, load_dropshot()))?;
     m.add(py, "load_hoops", py_fn!(py, load_hoops()))?;
     m.add(py, "set_gravity", py_fn!(py, set_gravity(gravity: PyDict)))?;
-    m.add(py, "tick", py_fn!(py, tick(time: PyFloat, ball: PyDict, car: PyDict)))?;
+    m.add(py, "tick", py_fn!(py, tick(packet: PyObject)))?;
+    m.add(py, "tick_dict", py_fn!(py, tick_dict(time: PyFloat, ball: PyDict, car: PyDict)))?;
     m.add(py, "get_slice", py_fn!(py, get_slice(time: PyFloat)))?;
     m.add(py, "get_data_for_shot_with_target", py_fn!(py, get_data_for_shot_with_target(target_left: PyTuple, target_right: PyTuple, time: PyFloat, py_index: PyInt)))?;
     m.add(py, "get_shot_with_target", py_fn!(py, get_shot_with_target(target_left: PyTuple, target_right: PyTuple, py_index: PyInt, all: PyBool)))?;
@@ -83,7 +84,143 @@ fn set_gravity(py: Python, py_gravity: PyDict) -> PyResult<PyObject> {
     Ok(py.None())
 }
 
-fn tick(py: Python, py_time: PyFloat, py_ball: PyDict, py_car: PyDict) -> PyResult<PyObject> {
+fn tick(py: Python, py_packet: PyObject) -> PyResult<PyObject> {
+    let mut game: &mut Game;
+    let cars: &mut Vec<Car>;
+
+    // maybe multithreading will actually become useful
+    // simulate max jump height
+    // simulate max double jump height
+    // add options to individual shots like min/max time
+    // add option for max path time
+
+    unsafe {
+        game = match GAME.as_mut() {
+            Some(game) => game,
+            None => {
+                return Err(PyErr::new::<exc::NameError, _>(py, NO_GAME_ERR));
+            }
+        };
+
+        cars = match &mut CARS {
+            Some(cars) => cars,
+            None => {
+                return Err(PyErr::new::<exc::NameError, _>(py, NO_CARS_ERR));
+            }
+        };
+    }
+
+    /*
+    Example GameTickPacket<
+        game_cars: [
+            PlayerInfo<
+                physics: Physics<
+                    location: Vector3<x: 0.0, y: -4608.0, z: 17.010000228881836>,
+                    rotation: Rotator<pitch: -0.009587380103766918, yaw: 1.5707963705062866, roll: 0.0>,
+                    velocity: Vector3<x: 0.0, y: 0.0, z: 0.210999995470047>,
+                    angular_velocity: Vector3<x: -0.0006099999882280827, y: 0.0, z: 0.0>
+                >,
+                score_info: ScoreInfo<score: 0, goals: 0, own_goals: 0, assists: 0, saves: 0, shots: 0, demolitions: 0>,
+                is_demolished: False,
+                has_wheel_contact: True,
+                is_super_sonic: False,
+                is_bot: True,
+                jumped: False,
+                double_jumped: False,
+                name: 'DownToEarth',
+                team: 0,
+                boost: 34,
+                hitbox: BoxShape<length: 118.00737762451172, width: 84.19940948486328, height: 36.15907287597656>,
+                hitbox_offset: Vector3<x: 13.875659942626953, y: 0.0, z: 20.754987716674805>,
+                spawn_id: 1793714700
+            >
+        ],
+        num_cars: 1,
+        game_boosts: <rlbot.utils.structures.game_data_struct.BoostPadState_Array_50 object at 0x000002C910DE8EC8>,
+        num_boost: 34,
+        game_ball: BallInfo<
+            physics: Physics<location: Vector3<x: 0.0, y: 0.0, z: 92.73999786376953>, rotation: Rotator<pitch: 0.0, yaw: 0.0, roll: 0.0>, velocity: Vector3<x: 0.0, y: 0.0, z: 0.0>, angular_velocity: Vector3<x: 0.0, y: 0.0, z: 0.0>>,
+            latest_touch: Touch<player_name: '', time_seconds: 0.0, hit_location: Vector3<x: 0.0, y: 0.0, z: 0.0>, hit_normal: Vector3<x: 0.0, y: 0.0, z: 0.0>, team: 0, player_index: 0>,
+            drop_shot_info: DropShotInfo<absorbed_force: 0.0, damage_index: 0, force_accum_recent: 0.0>,
+            collision_shape: CollisionShape<type: 1, box: BoxShape<length: 0.0, width: 0.0, height: 0.0>,
+            sphere: SphereShape<diameter: 182.49998474121094>, cylinder: CylinderShape<diameter: 0.0, height: 0.0>>
+        >,
+        game_info: GameInfo<seconds_elapsed: 718.4749755859375, game_time_remaining: -707.4849243164062, is_overtime: False, is_unlimited_time: True, is_round_active: False, is_kickoff_pause: False, is_match_ended: False, world_gravity_z: -650.0, game_speed: 0.0, frame_num: 86217>,
+        dropshot_tiles: <rlbot.utils.structures.game_data_struct.TileInfo_Array_200 object at 0x000002C910DE8EC8>,
+        num_tiles: 0,
+        teams: <rlbot.utils.structures.game_data_struct.TeamInfo_Array_2 object at 0x000002C910DE8EC8>,
+        num_teams: 2
+    >
+    */
+    let py_game_info = py_packet.getattr(py, "game_info")?;
+
+    game.ball.time = py_game_info.getattr(py, "seconds_elapsed")?.extract(py)?;
+
+    unsafe {
+        GAME_TIME = game.ball.time.clone();
+    }
+
+    let py_ball = py_packet.getattr(py, "game_ball")?;
+    let py_ball_physics = py_ball.getattr(py, "physics")?;
+
+    game.ball.location = get_vec3_named(py, py_ball_physics.getattr(py, "location")?)?;
+    game.ball.velocity = get_vec3_named(py, py_ball_physics.getattr(py, "velocity")?)?;
+    game.ball.angular_velocity = get_vec3_named(py, py_ball_physics.getattr(py, "angular_velocity")?)?;
+
+    let py_ball_shape = py_ball.getattr(py, "collision_shape")?;
+
+    game.ball.radius = py_ball_shape.getattr(py, "sphere")?.getattr(py, "diameter")?.extract::<f32>(py)? / 2.;
+    game.ball.collision_radius = game.ball.radius + 0.9;
+    game.ball.calculate_moi();
+
+    let py_game_cars = py_packet.getattr(py, "game_cars")?;
+    let num_cars = py_packet.getattr(py, "num_cars")?.extract::<usize>(py)?;
+
+    for i in 0..num_cars {
+        let py_car = py_game_cars.get_item(py, i)?;
+
+        let py_car_physics = py_car.getattr(py, "physics")?;
+
+        cars[i].location = get_vec3_named(py, py_car_physics.getattr(py, "location")?)?;
+        cars[i].velocity = get_vec3_named(py, py_car_physics.getattr(py, "velocity")?)?;
+        cars[i].angular_velocity = get_vec3_named(py, py_car_physics.getattr(py, "angular_velocity")?)?;
+
+        let py_car_rotator = py_car_physics.getattr(py, "rotation")?;
+
+        cars[i].pitch = py_car_rotator.getattr(py, "pitch")?.extract(py)?;
+        cars[i].yaw = py_car_rotator.getattr(py, "yaw")?.extract(py)?;
+        cars[i].roll = py_car_rotator.getattr(py, "roll")?.extract(py)?;
+
+        let py_car_hitbox = py_car.getattr(py, "hitbox")?;
+
+        cars[i].hitbox = Hitbox {
+            length: py_car_hitbox.getattr(py, "length")?.extract(py)?,
+            width: py_car_hitbox.getattr(py, "width")?.extract(py)?,
+            height: py_car_hitbox.getattr(py, "height")?.extract(py)?,
+        };
+
+        cars[i].hitbox_offset = get_vec3_named(py, py_car.getattr(py, "hitbox_offset")?)?;
+
+        cars[i].boost = py_car.getattr(py, "boost")?.extract(py)?;
+        cars[i].demolished = py_car.getattr(py, "is_demolished")?.extract(py)?;
+        cars[i].airborne = !py_car.getattr(py, "has_wheel_contact")?.extract(py)?;
+        cars[i].jumped = py_car.getattr(py, "jumped")?.extract(py)?;
+        cars[i].doublejumped = py_car.getattr(py, "double_jumped")?.extract(py)?;
+
+        cars[i].calculate_orientation_matrix();
+        cars[i].calculate_max_values();
+        cars[i].calculate_local_values();
+        cars[i].calculate_field();
+    }
+
+    unsafe {
+        BALL_STRUCT = Some(Ball::get_ball_prediction_struct_for_time(game, &6.));
+    }
+
+    Ok(py.None())
+}
+
+fn tick_dict(py: Python, py_time: PyFloat, py_ball: PyDict, py_car: PyDict) -> PyResult<PyObject> {
     let mut game: &mut Game;
     let cars: &mut Vec<Car>;
 
@@ -103,16 +240,6 @@ fn tick(py: Python, py_time: PyFloat, py_ball: PyDict, py_car: PyDict) -> PyResu
         };
     }
 
-    // adjust this to accept the entire game tick packet
-    // this will require a minor version bump
-    // make shot also take the car index
-    // maybe multithreading will actually become useful
-    // simulate max jump height
-    // simulate max double jump height
-    // add options to individual shots like min/max time
-    // add option to tick for max path time
-    // add option to change the TPS of the simulation
-
     game.ball.time = py_time.value(py) as f32;
 
     unsafe {
